/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file id_table.h
 *
 * Defines the cid_table class, which stores the message ids from recently
 * received messages (up to klifetime_secs seconds). There are methods for
 * adding a message id, looking up a message id, and purging message ids that
 * are older than the max lifetime.
 *
 * The cid_table class is implemented as a singleton. Use the instance method
 * to retrieve the cid_table instance.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _ID_TABLE_H
#define _ID_TABLE_H

#include "protocol.h"

#include <ctime>
#include <queue>
#include <map>
#include <string>
#include <iostream>
using namespace std;


/*****************************************************************************
 * FORWARD DECLARATIONS
 *****************************************************************************/
class cmsg;


/*****************************************************************************
 * CID_TABLE CLASS
 *****************************************************************************/
class cid_table {
  public:

  /** instance
   *
   * Returns the only instance of cid_table. This implements the singleton
   * pattern.
   */
  static cid_table &instance();

  /** add
   *
   * Adds a message id to the table. If the message id already exists in the
   * table, add does nothing.
   */
  void add(double id);

  /** contains
   *
   * Returns true if the table contains the given message id, false otherwise.
   */
  bool contains(double id) const;

  /** purge
   *
   * Purges all message ids that are older than klifetime_secs.
   */
  void purge();

  /** operator <<
   *
   * The stream insertion operator for cid_table - prints out the id table.
   *
   * NOTE: this is a *non*-member function so that the ostream can be the lhs
   * of the operator<<.
   */
  friend ostream &operator << (ostream &ostr, const cid_table &id_table);


  protected:
  /** ctor
   */
  cid_table();

  /** dtor
   */
  ~cid_table();


  /**************************************************************************
   * CONSTANTS AND MEMBERS
   **************************************************************************/
  static const uint kmid_lifetime_secs;  // max lifetime of an id

  map<double, clock_t> id_map;           // used for lookup
  queue< pair<double, clock_t> > id_q;   // used for purge
};

inline cid_table &cid_table::instance()
{
  static cid_table t;
  
  return t;
}

#endif  // _ID_TABLE_H



